from object import Object
from pygame.locals import *
import os

class Hero(Object):
    
    def __init__(self, x, y, image):
        import pygame
        
        super(Hero, self).__init__(x, y, image)
        self.shot = pygame.mixer.Sound(os.path.join("sound", "shot.wav"))
        self.shot.set_volume(0.15)
        self.flippedImage = pygame.image.load(os.path.join("image", "flip.png"))
        self.flipped = False
        self.flipParticleTick = 0
        self.flipDamageTick = 0
        self.flipSound = pygame.mixer.Sound(os.path.join("sound", "siren.wav"))
        self.flipSound.set_volume(0.2)
        self.keys = set()
        self.cooldown = 0.0
        self.health = 100
        self.wasHurt = 0.0
        self.hurtTick = 0.0
        self.hurtDir = 1
        self.jump = 0.0
        self.speed = 1.2
        
        self.loadHurtImages()
    
    def flip(self, state):
        was = self.flipped
        self.flipped = state
        
        if self.flipped:
            self.speed = 3.0
            if was == False:
                self.flipSound.play(-1)
        else:
            self.speed = 1.2
            self.flipSound.stop()
            
    def heal(self):
        self.health = min(100, self.health + 4)
        
    def loadHurtImages(self):
        self.hurt = []
        
        for number in range(0, 6):
            self.hurt.append(self.image.copy())
            for x in range(0, self.image.get_width()):
                for y in range(0, self.image.get_height()):
                    color = self.hurt[number].get_at((x, y))
                    color.r = min(color.r + (number + 2) * 40, 255)
                    self.hurt[number].set_at((x, y), color)
        
    def damage(self, ignore = False):
        if self.flipped and not ignore:
            return
        self.health = max(0, self.health - 2)
        self.wasHurt = 5.0
        
        if self.health == 0:
            from explosion import Explosion
            from main import Main
            center = [self.x + self.image.get_width() / 2.0, self.y + self.image.get_height() / 2.0]
            Main.objects.append(Explosion(center[0], center[1], 256, 80))
            Main.gameOver()
            self.flip(False)
        
    def keypressed(self, key, pressed):
        if pressed:
            self.keys.add(key)
        else:
            if key in self.keys:
                self.keys.remove(key)
            
    def shoot(self):
        from missile import Missile
        from main import Main
        import math, random
        
        center = [self.x + self.image.get_width() / 2.0, self.y + self.image.get_height() / 2.0]

        to = math.atan2(self.velocity[1], self.velocity[0])
        angles = (math.pi / (24.0 - random.random() * 8.0), 0, -math.pi / (24.0 - random.random() * 8.0))
        for angle in angles:
            nangle = to + angle
            new = Missile(center[0], center[1], center[0] + math.cos(nangle), center[1] + math.sin(nangle))
            new.speed = 10
            Main.objects.append(new)
        
        self.cooldown = 0.4
        self.shot.play()
            
    def update(self):
        super(Hero, self).update()
        
        from main import Main
                
        if self.flipped:
            self.flipParticleTick += 1
            self.flipDamageTick += 1
            if self.flipParticleTick > 7:
                self.flipParticleTick = 0
                from explosion import Explosion
                from enemy import Enemy
                import math
                center = [self.x + self.image.get_width() / 2.0, self.y + self.image.get_height() / 2.0]
                Main.objects.append(Explosion(center[0], center[1]))
                for object in Main.objects:
                    if not isinstance(object, Enemy):
                        continue
                    to = [object.x - center[0], object.y - center[1]]
                    len = math.sqrt(to[0] * to[0] + to[1] * to[1])
                    if len < 100:
                        Main.objects.append(Explosion(object.x, object.y))
                        object.kill()
            
            if self.flipDamageTick > 4:
                self.flipDamageTick = 0
                self.damage(True)
        
        self.jump += 0.2
        
        self.wasHurt = max(0, self.wasHurt - 0.1)
        self.cooldown = max(0, self.cooldown - 0.1)
        dspeed = self.speed * 0.7
        
        if K_LEFT in self.keys and K_UP in self.keys:
            self.velocity[0] -= dspeed
            self.velocity[1] -= dspeed
        elif K_LEFT in self.keys and K_DOWN in self.keys:
            self.velocity[0] -= dspeed
            self.velocity[1] += dspeed
        elif K_RIGHT in self.keys and K_DOWN in self.keys:
            self.velocity[0] += dspeed
            self.velocity[1] += dspeed
        elif K_RIGHT in self.keys and K_UP in self.keys:
            self.velocity[0] += dspeed
            self.velocity[1] -= dspeed
        elif K_LEFT in self.keys:
            self.velocity[0] -= self.speed
        elif K_RIGHT in self.keys:
            self.velocity[0] += self.speed
        elif K_UP in self.keys:
            self.velocity[1] -= self.speed
        elif K_DOWN in self.keys:
            self.velocity[1] += self.speed
        
        if not Main.buildMode:    
            if K_z in self.keys and self.cooldown == 0.0 and not self.flipped:
                self.shoot()
            
            self.flip(K_x in self.keys)
            
    def draw(self, window):
        img = self.image
        import math
        if self.wasHurt > 0:
            self.hurtTick = max(0, min(self.hurtTick + 0.3 * self.hurtDir, len(self.hurt) - 1))
            if self.hurtTick == len(self.hurt) - 1 or self.hurtTick == 0:
                self.hurtDir *= -1
            img = self.hurt[int(self.hurtTick)]
        if self.flipped:
            img = self.flippedImage
        window.blit(img, (self.x, self.y + math.sin(self.jump) * 2))