import pygame, sys, os
import pygame._view
from pygame.locals import *

from object import Object
from hero import Hero
from enemy import Enemy

sys.path.append('')

class Main(object):
    
    size = (640, 480)
    hero = None
    objects = []
    buildings = []
    blood = []
    bloodMap = None
    enemySpawnTimer = 30
    enemySpawnIncrease = 0
    gameIsOver = False
    buildMode = False
    gold = 0
    score = 0
    
    hpColor = pygame.Color(220, 0, 0)
    red = pygame.Color(255, 0, 0)
    white = pygame.Color(255, 255, 255)
    
    over = None
    build = None
    
    @staticmethod
    def addBlood(x, y):
        import random
        Main.bloodMap.blit(Main.blood[random.randrange(len(Main.blood))], (x, y))
        
    @staticmethod
    def drawText(window, text, color, size, target):
        font = pygame.font.Font(None, size) 
        todraw = font.render(text, 1, color)
        window.blit(todraw, target)
        
    @staticmethod
    def spawnEnemy():
        if Main.gameIsOver:
            return
        
        import random

        side = random.randrange(2)
        
        if side == 0:
            x = Main.size[0] * random.random()
            y = (0, 1)[random.randrange(2)] * Main.size[1]
        else:
            y = Main.size[1] * random.random()
            x = (0, 1)[random.randrange(2)] * Main.size[0]
        
        Main.objects.append(Enemy(Main.hero, x, y))
        
    @staticmethod
    def gameOver():
        from explosion import Explosion
        from missile import Missile
        for object in list(Main.objects):
            if isinstance(object, Explosion) or isinstance(object, Missile):
                continue
            object.kill()
            Main.objects.append(Explosion(object.x, object.y))
            
        for object in list(Main.buildings):
            object.kill()
            Main.objects.append(Explosion(object.x, object.y))
        
        Main.gameIsOver = True
        pygame.mixer.music.stop()
        Main.over.play()
        
    @staticmethod
    def addBuilding(building):
        Main.buildings.insert(0, building)
        
    @staticmethod
    def checkBuildingsAt(x, y):
        import math
        for object in Main.buildings:
            tocenter = [object.x + object.image.get_width() / 2.0, object.y + object.image.get_height() / 2.0]
            to = [x - tocenter[0], y - tocenter[1]]
            len = math.sqrt(to[0] * to[0] + to[1] * to[1])
            if len < 32:
                return True
        return False
    
    def loadBlood(self):
        for i in range(0, 5):
            Main.blood.append(pygame.image.load(os.path.join("image", "blood" + str(i) + ".png")))
        Main.bloodMap = pygame.Surface(Main.size, pygame.SRCALPHA, 32)
        Main.bloodMap.convert_alpha()
    
    def createTileMap(self):
        import random
        
        tiles = []
        for i in range(0, 3):
            tiles.append(pygame.image.load(os.path.join("image", "tile" + str(i) + ".png")))
        map = pygame.Surface(Main.size)
        for x in range(0, Main.size[0] / 16):
            for y in range(0, Main.size[1] / 16):
                map.blit(tiles[random.randrange(3)], (x * 16, y * 16))
        return map
    
    def drawInterface(self, window):
        healthPercent = Main.hero.health / 100.0
        barEnd = int(200.0 * healthPercent)
        window.fill(Main.hpColor, (10, 10, barEnd, 25))
        pygame.draw.rect(window, pygame.Color(0, 0, 0), (10, 10, 200, 25), 1)
        
        import random
        
        y = random.randrange(1, 2)
        while y < 24:
            x = 0
            range = int((1 - healthPercent) * 35)
            if range == 0:
                to = 0
            else:
                to = random.randrange(range)
            while x <= to:
                color = pygame.Color(220, int(x / 70.0 * 255), 0)
                window.set_at((10 + barEnd + x, 10 + y), color)
                x += random.randrange(1, 2)
            
            y += random.randrange(1, 2)
        
        scoresize = int(20 + (Main.score / 20000.0) * 20)
        Main.drawText(window, "Z - attack, X - FLIP, C - build", pygame.Color(255, 255, 255), 20, (10, 460))
        Main.drawText(window, str(Main.score), pygame.Color(190, 240, 50), scoresize, (300, 10))
        Main.drawText(window, "$" + str(Main.gold), pygame.Color(250, 220, 50), scoresize, (580, 10))
        
        if Main.buildMode:
            x = Main.hero.x + 35
            
            towerColor = Main.white
            if Main.gold < 70:
                towerColor = Main.red
            
            platformColor = Main.white
            if Main.gold < 110:
                platformColor = Main.red
            
            Main.drawText(window, "Z - tower ($70)", towerColor, 15, (x, Main.hero.y - 10))
            Main.drawText(window, "X - platform ($110)", platformColor, 15, (x, Main.hero.y + 5))
            Main.drawText(window, "C - cancel", Main.white, 15, (x, Main.hero.y + 20))
    
    def start(self):
        pygame.mixer.pre_init(44100, -16, 2, 2048)
        pygame.init()
        fpsClock = pygame.time.Clock()
        
        window = pygame.display.set_mode(Main.size)
        pygame.display.set_caption("flip.madness")
        
        tileMap = self.createTileMap()
        self.loadBlood()
        
        Main.hero = Hero(100, 100, pygame.image.load(os.path.join("image", "hero.png")))
        Main.objects.append(Main.hero)
        from tower import Tower
        from platform import Platform
        Main.addBuilding(Tower(300, 220))
        Main.addBuilding(Platform(330, 220))
        
        Main.over = pygame.mixer.Sound(os.path.join("sound", "over.wav"))
        Main.build = pygame.mixer.Sound(os.path.join("sound", "build.wav"))
        
        pygame.mixer.music.load(os.path.join("sound", "music.ogg"))
        pygame.mixer.music.set_volume(0.4)
        pygame.mixer.music.play()
        
        enemyTick = 0
        
        while True:
            if not Main.gameIsOver:
                Main.score += 1
            
            window.blit(tileMap, (0, 0))
            window.blit(Main.bloodMap, (0, 0))
            
            enemyTick += 1
            if enemyTick > Main.enemySpawnTimer:
                enemyTick = 0
                Main.spawnEnemy()
                
                if Main.enemySpawnTimer < -10:
                    for i in range(0, 10 + Main.enemySpawnTimer):
                        Main.spawnEnemy()
                
            Main.enemySpawnIncrease += 1
            if Main.enemySpawnIncrease > 250:
                Main.enemySpawnTimer -= 1
                Main.enemySpawnIncrease = 0
            
            if Main.hero.flipped:
                Main.enemySpawnIncrease += 2
            
            for object in list(Main.buildings):
                object.draw(window)
                object.update()
                if object.ondestroy:
                    Main.buildings.remove(object)
                    
            for object in list(Main.objects):
                object.draw(window)
                object.update()
                if object.ondestroy:
                    Main.objects.remove(object)
            
            self.drawInterface(window)
            
            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                elif event.type == KEYDOWN:
                    Main.hero.keypressed(event.key, True)
                    if event.key == K_ESCAPE:
                        pygame.event.post(pygame.event.Event(QUIT))
                elif event.type == KEYUP:
                    Main.hero.keypressed(event.key, False)
                    center = [Main.hero.x + Main.hero.image.get_width() / 2.0, Main.hero.y + Main.hero.image.get_height() / 2.0]
                    if event.key == K_c and not Main.gameIsOver:
                        Main.buildMode = not Main.buildMode
                    elif Main.buildMode and event.key == K_z:
                        if not Main.checkBuildingsAt(center[0], center[1]) and Main.gold >= 70:
                            Main.addBuilding(Tower(Main.hero.x, Main.hero.y))
                            Main.gold -= 70
                            Main.buildMode = False
                            Main.build.play()
                    elif Main.buildMode and event.key == K_x:
                        if not Main.checkBuildingsAt(center[0], center[1]) and Main.gold >= 110:
                            Main.addBuilding(Platform(Main.hero.x, Main.hero.y))
                            Main.gold -= 110
                            Main.buildMode = False
                            Main.build.play()
            
            pygame.display.update()
            fpsClock.tick(60)

Main().start()